<?php


/**


 * @package Admin


 */





if ( ! defined( 'WPSEO_VERSION' ) ) {


	header( 'HTTP/1.0 403 Forbidden' );


	die;


}





/**


 * Class that holds most of the admin functionality for WP SEO.


 */


class WPSEO_Admin {





	/**


	 * Class constructor


	 */


	function __construct() {


		$this->multisite_defaults();





		$options = get_seamlessseo_options();





		if ( isset( $options['stripcategorybase'] ) && $options['stripcategorybase'] ) {


			add_action( 'created_category', array( $this, 'schedule_rewrite_flush' ) );


			add_action( 'edited_category', array( $this, 'schedule_rewrite_flush' ) );


			add_action( 'delete_category', array( $this, 'schedule_rewrite_flush' ) );


		}





		if ( $this->grant_access() ) {


			add_action( 'admin_init', array( $this, 'options_init' ) );


			// Needs the lower than default priority so other plugins can hook underneath it without issue.


			add_action( 'admin_menu', array( $this, 'register_settings_page' ), 5 );


			





		//	add_filter( 'plugin_action_links', array( $this, 'add_action_link' ), 10, 2 );


			add_action( 'admin_print_scripts', array( $this, 'config_page_scripts' ) );


			





			if ( '0' == get_option( 'blog_public' ) )


				add_action( 'admin_footer', array( $this, 'blog_public_warning' ) );





			if ( isset( $options['meta_description_warning'] ) && true === $options['meta_description_warning'] ){


				//add_action( 'admin_footer', array( $this, 'meta_description_warning' ) );


			}


		}


 


		add_action( 'admin_init', array( $this, 'maybe_upgrade' ) );





		if ( isset( $options['cleanslugs'] ) && $options['cleanslugs'] )


			add_filter( 'name_save_pre', array( $this, 'remove_stopwords_from_slug' ), 0 );





		add_action( 'show_user_profile', array( $this, 'user_profile' ) );


		add_action( 'edit_user_profile', array( $this, 'user_profile' ) );


		add_action( 'personal_options_update', array( $this, 'process_user_option_update' ) );


		add_action( 'edit_user_profile_update', array( $this, 'process_user_option_update' ) );


		add_action( 'personal_options_update', array( $this, 'update_user_profile' ) );


		add_action( 'edit_user_profile_update', array( $this, 'update_user_profile' ) );





		add_filter( 'user_contactmethods', array( $this, 'update_contactmethods' ), 10, 1 );





		add_action( 'update_option_seamlessseo_titles', array( $this, 'clear_cache' ) );


		add_action( 'update_option_seamlessseo', array( $this, 'clear_cache' ) );





		add_action( 'update_option_seamlessseo_permalinks', array( $this, 'clear_rewrites' ) );


		add_action( 'update_option_seamlessseo_xml', array( $this, 'clear_rewrites' ) );





		add_action( 'after_switch_theme', array( $this, 'switch_theme' ) );


		add_action( 'switch_theme', array( $this, 'switch_theme' ) );


		


		// galink


		if(is_admin()) {


			


			add_action( 'admin_init', array($this, 'galink_register_styles'));


			add_action( 'admin_print_styles', array($this, 'galink_enqueue_styles'));


			add_action( 'wp_print_scripts', array($this, 'galink_enqueue_scripts') );


	    }else{


			add_action('wp_head', array($this, 'galink_head_output_authorship') );


			add_action('wp_head', array($this, 'galink_head_output_publisher') );


		}





		


	}





	/**


	 * Schedules a rewrite flush to happen at shutdown


	 */


	function schedule_rewrite_flush() {


		add_action( 'shutdown', 'flush_rewrite_rules' );


	}





	/**


	 * Clears the cache


	 */


	function clear_cache() {


		if ( function_exists( 'w3tc_pgcache_flush' ) ) {


			w3tc_pgcache_flush();


		}


		else if ( function_exists( 'wp_cache_clear_cache' ) ) {


			wp_cache_clear_cache();


		}


	}





	/**


	 * Clear rewrites


	 */


	function clear_rewrites() {


		delete_option( 'rewrite_rules' );


	}





	/**


	 * Register all the options needed for the configuration pages.


	 */


	function options_init() {


		register_setting( 'yoast_seamlessseo_options', 'seamlessseo' );


		register_setting( 'yoast_seamlessseo_permalinks_options', 'seamlessseo_permalinks' );


		register_setting( 'yoast_seamlessseo_titles_options', 'seamlessseo_titles' );


		register_setting( 'yoast_seamlessseo_rss_options', 'seamlessseo_rss' );


		register_setting( 'yoast_seamlessseo_internallinks_options', 'seamlessseo_internallinks' );


		register_setting( 'yoast_seamlessseo_xml_sitemap_options', 'seamlessseo_xml' );


		register_setting( 'yoast_seamlessseo_social_options', 'seamlessseo_social' );


		register_setting('wpanl_settings_fields', 'wpanl_options');


		if ( function_exists( 'is_multisite' ) && is_multisite() ) {


			if ( get_option( 'seamlessseo' ) == '1pseo_social' )


				delete_option( 'seamlessseo' );


			register_setting( 'yoast_seamlessseo_multisite_options', 'seamlessseo_ms' );


		}


		


		// google plus settings


		register_setting( 'galink-settings', 'galink_options' );


		register_setting( 'galink-settings', 'galink_google_publisher_profile' );


		register_setting( 'galink-settings', 'galink_exclude_post_categories' );


		register_setting( 'galink-settings', 'galink_exclude_custom_post_type' );


		register_setting( 'galink-settings', 'galink_remove_authorship_from_all_pages' );				


		register_setting( 'galink-settings', 'galink_exclude_pages' );		


		


	}





	function multisite_defaults() {


		$option = get_option( 'seamlessseo' );


		if ( function_exists( 'is_multisite' ) && is_multisite() && ! is_array( $option ) ) {


			$options = get_site_option( 'seamlessseo_ms' );


			if ( is_array( $options ) && isset( $options['defaultblog'] ) && ! empty( $options['defaultblog'] ) && $options['defaultblog'] != 0 ) {


				foreach ( get_seamlessseo_options_arr() as $seamlessseo_option ) {


					update_option( $seamlessseo_option, get_blog_option( $options['defaultblog'], $seamlessseo_option ) );


				}


			}


			$option['ms_defaults_set'] = true;


			update_option( 'seamlessseo', $option );


		}


	}





	/**


	 * Check whether the current user is allowed to access the configuration.


	 *


	 * @return boolean


	 */


	function grant_access() {


		if ( ! function_exists( 'is_multisite' ) || ! is_multisite() )


			return true;





		$options = get_site_option( 'seamlessseo_ms' );


		if ( ! is_array( $options ) || ! isset( $options['access'] ) )


			return true;





		if ( $options['access'] == 'superadmin' && ! is_super_admin() )


			return false;





		return true;


	}





	/**


	 * Register the menu item and its sub menu's.


	 *


	 * @global array $submenu used to change the label on the first item.


	 */


	function register_settings_page() {


		add_action('admin_menu', array($this,'auto_seo_links_menu'));


		add_action('admin_menu', array($this,'google_plus_links_menu'));


		


		global $submenu;


		if ( isset( $submenu['seamlessseo_dashboard'] ) )


			$submenu['seamlessseo_dashboard'][0][0] = __( 'Dashboard', 'wordpress-seo' );


	}





	function auto_seo_links_menu() {


	add_options_page('Seamless SEO', 'Seamless SEO', 'manage_options','auto-nofollow-links-admin', array($this,'seo_setting_page'));


	}





	function google_plus_links_menu() {


	add_options_page('Google Author Link', 'Google Author Link', 'manage_options','google-author-link', array($this,'gplus_profile_setting_page'));


}


 





function seo_setting_page() {


if ( isset( $_GET['page'] ) && 'auto-nofollow-links-admin' == $_GET['page'] )


require_once( WPSEO_PATH . 'admin/pages/auto-nofollow-links-admin.php' ); 


} 





function gplus_profile_setting_page() {


if ( isset( $_GET['page'] ) && 'google-author-link' == $_GET['page'] )


require_once(WPSEO_PATH.'inc/google-author-link-options.php');


require_once( WPSEO_PATH . 'admin/pages/google-author-link.php' );


galink_options_show(); 


} 











	/**


	 * Loads the form for the Dashboard page.


	 */


	function config_page() {


		if ( isset( $_GET['page'] ) && 'seamlessseo_dashboard' == $_GET['page'] )


			require_once( WPSEO_PATH . 'admin/pages/dashboard.php' );


	}





	





	/**


	 * Display an error message when the blog is set to private.


	 */


	function blog_public_warning() {


		if ( function_exists( 'is_network_admin' ) && is_network_admin() )


			return;





		$options = get_option( 'seamlessseo' );


		if ( isset( $options['ignore_blog_public_warning'] ) && $options['ignore_blog_public_warning'] == 'ignore' )


			return;


		echo "<div id='message' class='error'>";


		echo "<p><strong>" . __( "Huge SEO Issue: You're blocking access to robots.", 'wordpress-seo' ) . "</strong> " . sprintf( __( "You must %sgo to your Reading Settings%s and uncheck the box for Search Engine Visibility.", 'wordpress-seo' ), "<a href='" . admin_url( 'options-reading.php' ) . "'>", "</a>" ) . " <a href='javascript:seamlessseo_setIgnore(\"blog_public_warning\",\"message\",\"" . wp_create_nonce( 'seamlessseo-ignore' ) . "\");' class='button'>" . __( "I know, don't bug me.", 'wordpress-seo' ) . "</a></p></div>";


	}





	/**


	 * Display an error message when the theme contains a meta description tag.


	 *


	 * @since 1.4.14


	 */


	function meta_description_warning() {


		if ( function_exists( 'is_network_admin' ) && is_network_admin() )


			return;





		// No need to double display it on the dashboard


		if ( isset( $_GET['page'] ) && 'seamlessseo_dashboard' == $_GET['page'] )


			return;





		$options = get_option( 'seamlessseo' );


		if ( isset( $options['ignore_meta_description_warning'] ) && 'ignore' === $options['ignore_meta_description_warning'] )


			return;





		echo '<div id="metamessage" class="error">';


		echo '<p><strong>' . __( 'SEO Issue:', 'wordpress-seo' ) . '</strong> ' . sprintf( __( 'Your theme contains a meta description, which blocks WordPress SEO from working properly. Please visit the %sSEO Dashboard%s to fix this.', 'wordpress-seo' ), '<a href="' . admin_url( 'admin.php?page=seamlessseo_dashboard' ) . '">', '</a>' ) . ' <a href="javascript:seamlessseo_setIgnore(\'meta_description_warning\',\'metamessage\',\'' . wp_create_nonce( 'seamlessseo-ignore' ) . '\');" class="button">' . __( "I know, don't bug me.", 'wordpress-seo' ) . '</a></p></div>';


	}





	/**


	 * Add a link to the settings page to the plugins list


	 *


	 * @staticvar string $this_plugin holds the directory & filename for the plugin


	 *


	 * @param array  $links array of links for the plugins, adapted when the current plugin is found.


	 * @param string $file  the filename for the current plugin, which the filter loops through.


	 *


	 * @return array $links


	 */


	function add_action_link( $links, $file ) {


		static $this_plugin;


		if ( empty( $this_plugin ) ) $this_plugin = 'wordpress-seo/seamless-seo.php';


		if ( $file == $this_plugin ) {


			// $settings_link = '<a href="' . admin_url( 'admin.php?page=seamlessseo_dashboard' ) . '">' . __( 'Settings', 'wordpress-seo' ) . '</a>';


			$settings_link ='';


			array_unshift( $links, $settings_link );


		}


		return $links;


	}





	/**


	 * Enqueues the (tiny) global JS needed for the plugin.


	 */


	function config_page_scripts() {


		wp_enqueue_script( 'seamlessseo-admin-global-script', plugins_url( 'js/seamless-seo-admin-global.js', dirname( __FILE__ ) ), array( 'jquery' ), WPSEO_VERSION, true );


	}





	/**


	 * Updates the user metas that (might) have been set on the user profile page.


	 *


	 * @param int $user_id of the updated user


	 */


	function process_user_option_update( $user_id ) {


		if ( isset( $_POST['seamlessseo_author_title'] ) ) {


			check_admin_referer( 'seamlessseo_user_profile_update', 'seamlessseo_nonce' );


			update_user_meta( $user_id, 'seamlessseo_title', ( isset( $_POST['seamlessseo_author_title'] ) ? esc_html( $_POST['seamlessseo_author_title'] ) : '' ) );


			update_user_meta( $user_id, 'seamlessseo_metadesc', ( isset( $_POST['seamlessseo_author_metadesc'] ) ? esc_html( $_POST['seamlessseo_author_metadesc'] ) : '' ) );


			update_user_meta( $user_id, 'seamlessseo_metakey', ( isset( $_POST['seamlessseo_author_metakey'] ) ? esc_html( $_POST['seamlessseo_author_metakey'] ) : '' ) );


		}


	}





	/**


	 * Filter the $contactmethods array and add Facebook, Google+ and Twitter.


	 *


	 * These are used with the Facebook author, rel="author" and Twitter cards implementation.


	 *


	 * @param array $contactmethods currently set contactmethods.


	 *


	 * @return array $contactmethods with added contactmethods.


	 */


	function update_contactmethods( $contactmethods ) {


		// Add Google+


		$contactmethods['googleplus'] = __( "Google+", 'wordpress-seo' );


		// Add Twitter


		$contactmethods['twitter'] = __( 'Twitter username (without @)', 'wordpress-seo' );


		// Add Facebook


		$contactmethods['facebook'] = __( 'Facebook profile URL', 'wordpress-seo' );





		return $contactmethods;


	}





	function galink_register_styles(){


		wp_register_style('google_author_link_admin_css', plugins_url('css/google-author-link-admin.css', __FILE__));


	}


	


	function galink_enqueue_styles(){


		wp_enqueue_style( 'google_author_link_admin_css' );


	}


	function galink_enqueue_scripts(){


		wp_enqueue_script( 'google-author-link', plugin_dir_url( __FILE__ ) . 'js/google-author-link-admin.js', array( 'jquery' ) );


	}


	


	function galink_google_profile(){


		// Add Google Profiles


		$galink_contactmethods['galink_profile'] = 'Google Profile URL';


		return $galink_contactmethods;


	}


	


	function galink_head_output_authorship(){


		global $post;


		


		//check if exclude all pages


		$remove_all_pages = get_option('galink_remove_authorship_from_all_pages', 0);


		if( $remove_all_pages ){


			return;


		}


		//check if is exclude page


		$exclude_pages = get_option('galink_exclude_pages', '');


		if( $exclude_pages && is_array($exclude_pages) && count($exclude_pages) > 0 && in_array($post->ID, $exclude_pages) ){


			return;


		}


		//check if is exclude category


		$exclude_category = get_option('galink_exclude_post_categories', ''); 


		if( $exclude_category && is_array($exclude_category) && count($exclude_category) > 0 && in_category( $exclude_category ) ){


			return;


		}


		


		//check if is exclude post type


		$exclude_post_type = get_option('galink_exclude_custom_post_type', '');


		if( $exclude_post_type && is_array($exclude_post_type) && count($exclude_post_type) > 0 && is_singular( $exclude_post_type ) ){


			return;


		}


		


		//check if home page


		if ( is_home() || is_front_page() ){


			$galink_home_user = get_option('galink_options', 0);


			if ($galink_home_user < 1) return;


			


			$galink_author_id = $galink_home_user;


		}else if ( is_single() || is_page() ){


			$galink_author_id = $post->post_author;


		}


		$galink_google_profile = get_user_meta($galink_author_id, 'galink_profile', true);


		if ( $galink_google_profile ){


			echo '<link rel="author" href="'.$galink_google_profile.'"/>'."\n";


		}


	}


	


	


	function galink_head_output_publisher(){


		$galink_google_publisher_profile = get_option('galink_google_publisher_profile', '');


		if ( $galink_google_publisher_profile ){


			echo '<link rel="publisher" href="'.$galink_google_publisher_profile.'"/>'."\n";


		}


	}


	


	


	/**


	 * Add the inputs needed for SEO values to the User Profile page


	 *


	 * @param object $user


	 */


	function user_profile( $user ) {





		if ( ! current_user_can( 'edit_users' ) )


			return;





		$options = get_seamlessseo_options();





		wp_nonce_field( 'seamlessseo_user_profile_update', 'seamlessseo_nonce' );


		?>


		<h3 id="wordpress-seo"><?php _e( "WordPress SEO settings", 'wordpress-seo' ); ?></h3>


		<table class="form-table">


			<tr>


				<th><?php _e( "Title to use for Author page", 'wordpress-seo' ); ?></th>


				<td><input class="regular-text" type="text" name="seamlessseo_author_title"


									 value="<?php echo esc_attr( get_the_author_meta( 'seamlessseo_title', $user->ID ) ); ?>" /></td>


			</tr>


			<tr>


				<th><?php _e( "Meta description to use for Author page", 'wordpress-seo' ); ?></th>


				<td><textarea rows="3" cols="30"


											name="seamlessseo_author_metadesc"><?php echo esc_html( get_the_author_meta( 'seamlessseo_metadesc', $user->ID ) ); ?></textarea>


				</td>


			</tr>


			<?php if ( isset( $options['usemetakeywords'] ) && $options['usemetakeywords'] ) { ?>


				<tr>


					<th><?php _e( "Meta keywords to use for Author page", 'wordpress-seo' ); ?></th>


					<td><input class="regular-text" type="text" name="seamlessseo_author_metakey"


										 value="<?php echo esc_attr( get_the_author_meta( 'seamlessseo_metakey', $user->ID ) ); ?>" /></td>


				</tr>


			<?php } ?>


		</table>


		<br /><br />


	<?php


	}





	/**


	 * Determine whether the seamlessseo option holds the current version, if it doesn't, run


	 * the upgrade procedures.


	 */


	function maybe_upgrade() {


		$options         = get_option( 'seamlessseo' );


		$current_version = isset( $options['version'] ) ? $options['version'] : 0;





		if ( version_compare( $current_version, WPSEO_VERSION, '==' ) )


			return;





		// <= 0.3.5: flush rewrite rules for new XML sitemaps


		if ( $current_version == 0 ) {


			$this->schedule_rewrite_flush();


		}





		if ( version_compare( $current_version, '0.4.2', '<' ) ) {


			$xml_opt = array();


			// Move XML Sitemap settings from general array to XML specific array, general settings first


			foreach ( array( 'enablexmlsitemap', 'xml_include_images', 'xml_ping_google', 'xml_ping_bing', 'xml_ping_yahoo', 'xml_ping_ask', 'xmlnews_posttypes' ) as $opt ) {


				if ( isset( $options[$opt] ) ) {


					$xml_opt[$opt] = $options[$opt];


					unset( $options[$opt] );


				}


			}


			// Per post type settings


			foreach ( get_post_types() as $post_type ) {


				if ( in_array( $post_type, array( 'revision', 'nav_menu_item', 'attachment' ) ) )


					continue;





				if ( isset( $options['post_types-' . $post_type . '-not_in_sitemap'] ) ) {


					$xml_opt['post_types-' . $post_type . '-not_in_sitemap'] = $options['post_types-' . $post_type . '-not_in_sitemap'];


					unset( $options['post_types-' . $post_type . '-not_in_sitemap'] );


				}


			}


			// Per taxonomy settings


			foreach ( get_taxonomies() as $taxonomy ) {


				if ( in_array( $taxonomy, array( 'nav_menu', 'link_category', 'post_format' ) ) )


					continue;





				if ( isset( $options['taxonomies-' . $taxonomy . '-not_in_sitemap'] ) ) {


					$xml_opt['taxonomies-' . $taxonomy . '-not_in_sitemap'] = $options['taxonomies-' . $taxonomy . '-not_in_sitemap'];


					unset( $options['taxonomies-' . $taxonomy . '-not_in_sitemap'] );


				}


			}


			if ( get_option( 'seamlessseo_xml' ) === false )


				update_option( 'seamlessseo_xml', $xml_opt );


			unset( $xml_opt );





			// Clean up other no longer used settings


			unset( $options['seamlessseodir'], $options['seamlessseourl'] );


		}





		if ( version_compare( $current_version, '1.0.2.2', '<' ) ) {


			$opt = (array) get_option( 'seamlessseo_indexation' );


			unset( $opt['hideindexrel'], $opt['hidestartrel'], $opt['hideprevnextpostlink'], $opt['hidewpgenerator'] );


			update_option( 'seamlessseo_indexation', $opt );


		}





		if ( version_compare( $current_version, '1.0.4', '<' ) ) {


			$opt    = (array) get_option( 'seamlessseo_indexation' );


			$newopt = array(


				'opengraph'  => isset( $opt['opengraph'] ) ? $opt['opengraph'] : '',


				'fb_adminid' => isset( $opt['fb_adminid'] ) ? $opt['fb_adminid'] : '',


				'fb_appid'   => isset( $opt['fb_appid'] ) ? $opt['fb_appid'] : '',


			);


			update_option( 'seamlessseo_social', $newopt );


			unset( $opt['opengraph'], $opt['fb_pageid'], $opt['fb_adminid'], $opt['fb_appid'] );


			update_option( 'seamlessseo_indexation', $opt );


		}





		if ( version_compare( $current_version, '1.2', '<' ) ) {


			$opt     = get_option( 'seamlessseo_indexation' );


			$metaopt = get_option( 'seamlessseo_titles' );





			$metaopt['noindex-author']      = isset( $opt['noindexauthor'] ) ? $opt['noindexauthor'] : '';


			$metaopt['disable-author']      = isset( $opt['disableauthor'] ) ? $opt['disableauthor'] : '';


			$metaopt['noindex-archive']     = isset( $opt['noindexdate'] ) ? $opt['noindexdate'] : '';


			$metaopt['noindex-category']    = isset( $opt['noindexcat'] ) ? $opt['noindexcat'] : '';


			$metaopt['noindex-post_tag']    = isset( $opt['noindextag'] ) ? $opt['noindextag'] : '';


			$metaopt['noindex-post_format'] = isset( $opt['noindexpostformat'] ) ? $opt['noindexpostformat'] : '';


			$metaopt['noindex-subpages']    = isset( $opt['noindexsubpages'] ) ? $opt['noindexsubpages'] : '';


			$metaopt['hide-rsdlink']        = isset( $opt['hidersdlink'] ) ? $opt['hidersdlink'] : '';


			$metaopt['hide-feedlinks']      = isset( $opt['hidefeedlinks'] ) ? $opt['hidefeedlinks'] : '';


			$metaopt['hide-wlwmanifest']    = isset( $opt['hidewlwmanifest'] ) ? $opt['hidewlwmanifest'] : '';


			$metaopt['hide-shortlink']      = isset( $opt['hideshortlink'] ) ? $opt['hideshortlink'] : '';





			update_option( 'seamlessseo_titles', $metaopt );





			delete_option( 'seamlessseo_indexation' );





			seamlessseo_title_test();


		}





		// Clean up the wrong seamlessseo options


		if ( version_compare( $current_version, '1.2.3', '<' ) ) {


			$opt = get_option( 'seamlessseo' );





			if ( is_array( $opt ) ) {


				foreach ( $opt as $key => $val ) {


					if ( ! in_array( $key, array( 'ignore_blog_public_warning', 'ignore_tour', 'ignore_page_comments', 'ignore_permalink', 'ms_defaults_set', 'version', 'disableadvanced_meta', 'googleverify', 'msverify', 'alexaverify' ) ) ) {


						unset( $opt[$key] );


					}


				}





				update_option( 'seamlessseo', $opt );


				unset( $opt );


			}


		}





		// Fix wrongness created by buggy version 1.2.2


		if ( version_compare( $current_version, '1.2.4', '<' ) ) {


			$options = get_option( 'seamlessseo_titles' );


			if ( is_array( $options ) && isset( $options['title-home'] ) && $options['title-home'] == '%%sitename%% - %%sitedesc%% - 12345' ) {


				$options['title-home'] = '%%sitename%% - %%sitedesc%%';


				update_option( 'seamlessseo_titles', $options );


			}


		}





		if ( version_compare( $current_version, '1.2.8', '<' ) ) {


			$options = get_option( 'seamlessseo' );


			if ( is_array( $options ) && isset( $options['presstrends'] ) ) {


				$options['yoast_tracking'] = 'on';


				unset( $options['presstrends'] );


				update_option( 'seamlessseo', $options );


			}


		}





		if ( version_compare( $current_version, '1.2.8.2', '<' ) ) {


			$options = get_option( 'seamlessseo' );


			if ( is_array( $options ) && isset( $options['presstrends'] ) ) {


				$options['yoast_tracking'] = 'on';


				unset( $options['presstrends'] );


			}


			if ( is_array( $options ) && isset( $options['presstrends_popup'] ) ) {


				$options['tracking_popup'] = 'on';


				unset( $options['presstrends_popup'] );


			}


			update_option( 'seamlessseo', $options );


		}





		if ( version_compare( $current_version, '1.3.2', '<' ) ) {


			$options = get_option( 'seamlessseo_xml' );





			$options['post_types-attachment-not_in_sitemap'] = true;


			update_option( 'seamlessseo_xml', $options );


		}





		if ( version_compare( $current_version, '1.4.13', '<' ) ) {


			seamlessseo_description_test();


		}





		if ( version_compare( $current_version, '1.4.15', '<' ) ) {


			$this->schedule_rewrite_flush();


		}





		if ( version_compare( $current_version, '1.4.16', '<' ) ) {


			$options = get_option( 'seamlessseo_permalinks' );





			if ( ! is_array( $options ) )


				$options = array();





			$options['cleanslugs'] = 'on';


			update_option( 'seamlessseo_permalinks', $options );


		}





		$options            = get_option( 'seamlessseo' );


		$options['version'] = WPSEO_VERSION;


		update_option( 'seamlessseo', $options );


	}





	/**


	 * Cleans stopwords out of the slug, if the slug hasn't been set yet.


	 *


	 * @since 1.1.7


	 *


	 * @param string $slug if this isn't empty, the function will return an unaltered slug.


	 *


	 * @return string $clean_slug cleaned slug


	 */


	function remove_stopwords_from_slug( $slug ) {


		// Don't change an existing slug


		if ( isset( $slug ) && $slug !== '' )


			return $slug;





		if ( ! isset( $_POST['post_title'] ) )


			return $slug;





		// Lowercase the slug and strip slashes


		$clean_slug = sanitize_title( stripslashes( $_POST['post_title'] ) );





		// Turn it to an array and strip stopwords by comparing against an array of stopwords


		$clean_slug_array = array_diff( explode( '-', $clean_slug ), $this->stopwords() );





		// Turn the sanitized array into a string


		$clean_slug = join( '-', $clean_slug_array );





		return $clean_slug;


	}





	/**


	 * Returns the stopwords for the current language


	 *


	 * @since 1.1.7


	 *


	 * @return array $stopwords array of stop words to check and / or remove from slug


	 */


	function stopwords() {


		/* translators: this should be an array of stopwords for your language, separated by comma's. */


		$stopwords = explode( ',', __( "a,about,above,after,again,against,all,am,an,and,any,are,aren't,as,at,be,because,been,before,being,below,between,both,but,by,can't,cannot,could,couldn't,did,didn't,do,does,doesn't,doing,don't,down,during,each,few,for,from,further,had,hadn't,has,hasn't,have,haven't,having,he,he'd,he'll,he's,her,here,here's,hers,herself,him,himself,his,how,how's,i,i'd,i'll,i'm,i've,if,in,into,is,isn't,it,it's,its,itself,let's,me,more,most,mustn't,my,myself,no,nor,not,of,off,on,once,only,or,other,ought,our,ours,ourselves,out,over,own,same,shan't,she,she'd,she'll,she's,should,shouldn't,so,some,such,than,that,that's,the,their,theirs,them,themselves,then,there,there's,these,they,they'd,they'll,they're,they've,this,those,through,to,too,under,until,up,very,was,wasn't,we,we'd,we'll,we're,we've,were,weren't,what,what's,when,when's,where,where's,which,while,who,who's,whom,why,why's,with,won't,would,wouldn't,you,you'd,you'll,you're,you've,your,yours,yourself,yourselves", "wordpress-seo" ) );





		/**


		 * Allows filtering of the stop words list


		 * Especially useful for users on a language in which WPSEO is not available yet


		 * and/or users who want to turn off stop word filtering


		 * @api  array  $stopwords  Array of all lowercase stopwords to check and/or remove from slug


		 */


		$stopwords = apply_filters( 'seamlessseo_stopwords', $stopwords );





		return $stopwords;


	}





	/**


	 * Check whether the stopword appears in the string


	 *


	 * @param string $haystack    The string to be checked for the stopword


	 * @param bool   $checkingUrl Whether or not we're checking a URL


	 *


	 * @return bool|mixed


	 */


	function stopwords_check( $haystack, $checkingUrl = false ) {


		$stopWords = $this->stopwords();





		if ( is_array( $stopWords ) && count( $stopWords ) > 0 ) {


			foreach ( $stopWords as $stopWord ) {


				// If checking a URL remove the single quotes


				if ( $checkingUrl )


					$stopWord = str_replace( "'", '', $stopWord );





				// Check whether the stopword appears as a whole word


				// @todo check whether the use of \b (=word boundary) would be more efficient ;-)


				$res = preg_match( "`(^|[ \n\r\t\.,'\(\)\"\+;!?:])" . preg_quote( $stopWord, '`' ) . "($|[ \n\r\t\.,'\(\)\"\+;!?:])`iu", $haystack, $match );


				if ( $res > 0 )


					return $stopWord;


			}


		}





		return false;


	}





	/**


	 * Log the timestamp when a user profile has been updated


	 */


	function update_user_profile( $user_id ) {


		if ( current_user_can( 'edit_user', $user_id ) ) {


			update_user_meta( $user_id, '_yoast_seamlessseo_profile_updated', time() );


		}


	}





	/**


	 * Log the updated timestamp for user profiles when theme is changed


	 */


	function switch_theme() {


		foreach ( get_users( array( 'who' => 'authors' ) ) as $user ) {


			update_user_meta( $user->ID, '_yoast_seamlessseo_profile_updated', time() );


		}


	}





}





// Globalize the var first as it's needed globally.


global $seamlessseo_admin;


$seamlessseo_admin = new WPSEO_Admin();	





/******************************************************/





// post type permalinks





class Custom_Post_Type_Permalinks {





	public $version = "0.9";





	public $default_structure = '/%postname%/';





	/**


	 *


	 * Add Action & filter hooks.


	 *


	 */


	public function add_hook () {





		add_action( 'plugins_loaded', array(&$this,'check_version') );


		add_action( 'wp_loaded', array(&$this,'add_archive_rewrite_rules'), 99 );


		add_action( 'wp_loaded', array(&$this,'add_tax_rewrite_rules') );


		add_action( 'wp_loaded', array(&$this, "dequeue_flush_rules"),100);


		add_action( 'parse_request', array(&$this, "parse_request") );


		add_action( 'registered_post_type', array(&$this,'registered_post_type'), 10, 2 );








		if(get_option( "permalink_structure") != "") {


			add_filter( 'post_type_link', array(&$this,'post_type_link'), 10, 4 );


			add_filter( 'getarchives_join', array(&$this,'getarchives_join'), 10, 2 ); // [steve]


			add_filter( 'getarchives_where', array(&$this,'getarchives_where'), 10 , 2 );


			add_filter( 'get_archives_link', array(&$this,'get_archives_link'), 20, 1 );


			add_filter( 'term_link', array(&$this,'term_link'), 10, 3 );


			add_filter( 'attachment_link', array(&$this, 'attachment_link'), 20 , 2);


		}








		add_action( 'init', array(&$this,'load_textdomain') );


		add_action( 'init', array(&$this, 'update_rules') );


		add_action( 'update_option_cptp_version', array(&$this, 'update_rules') );


		add_action( 'admin_init', array(&$this,'settings_api_init'), 30 );


		add_action( 'admin_enqueue_scripts', array(&$this,'enqueue_css_js') );


		add_action( 'admin_footer', array(&$this,'pointer_js') );








	}





	/**


	 *


	 * dequeue flush rules


	 * @since 0.9


	 *


	 */





	public function dequeue_flush_rules () {


		if(get_option("queue_flush_rules")){


			flush_rewrite_rules();


			update_option( "queue_flush_rules", 0 );





		}


	}








	/**


	 *


	 * dequeue flush rules


	 * @since 0.8.6


	 *


	 */





	public function check_version() {


		$version = get_option('cptp_version', 0);


		if($version != $this->version) {


			update_option('cptp_version', $this->version);


		}


	}





	/**


	 *


	 * Get Custom Taxonomies parents.


	 * @version 1.0


	 *


	 */


	private function get_taxonomy_parents( $id, $taxonomy = 'category', $link = false, $separator = '/', $nicename = false, $visited = array() ) {


		$chain = '';


		$parent = &get_term( $id, $taxonomy, OBJECT, 'raw');


		if ( is_wp_error( $parent ) ) {


			return $parent;


		}





		if ( $nicename ){


			$name = $parent->slug;


		}else {


			$name = $parent->name;


		}





		if ( $parent->parent && ( $parent->parent != $parent->term_id ) && !in_array( $parent->parent, $visited ) ) {


			$visited[] = $parent->parent;


			$chain .= $this->get_taxonomy_parents( $parent->parent, $taxonomy, $link, $separator, $nicename, $visited );


		}





		if ( $link ) {


			$chain .= '<a href="' . get_term_link( $parent->term_id, $taxonomy ) . '" title="' . esc_attr( sprintf( __( "View all posts in %s" ), $parent->name ) ) . '">'.$name.'</a>' . $separator;


		}else {


			$chain .= $name.$separator;


		}


		return $chain;


	}











	/**


	 *


	 * Add rewrite rules for archives.


	 * @version 1.1


	 *


	 */


	public function add_archive_rewrite_rules() {


		$post_types = get_post_types( array('_builtin'=>false, 'publicly_queryable'=>true,'show_ui' => true) );





		foreach ( $post_types as $post_type ):


			if( !$post_type ) {


				continue;


			}





			$permalink = get_option( $post_type.'_structure' );


			$post_type_obj = get_post_type_object($post_type);


			$slug = $post_type_obj->rewrite['slug'];


			if( !$slug )


				$slug = $post_type;





			if( $post_type_obj->has_archive ){


				if( is_string( $post_type_obj->has_archive ) ){


					$slug = $post_type_obj->has_archive;


				};








				add_rewrite_rule( $slug.'/date/([0-9]{4})/([0-9]{1,2})/([0-9]{1,2})/feed/(feed|rdf|rss|rss2|atom)/?$', 'index.php?year=$matches[1]&monthnum=$matches[2]&day=$matches[3]&feed=$matches[4]&post_type='.$post_type, 'top' );


				add_rewrite_rule( $slug.'/date/([0-9]{4})/([0-9]{1,2})/([0-9]{1,2})/(feed|rdf|rss|rss2|atom)/?$', 'index.php?year=$matches[1]&monthnum=$matches[2]&day=$matches[3]&feed=$matches[4]&post_type='.$post_type, 'top' );


				add_rewrite_rule( $slug.'/date/([0-9]{4})/([0-9]{1,2})/([0-9]{1,2})/page/?([0-9]{1,})/?$', 'index.php?year=$matches[1]&monthnum=$matches[2]&day=$matches[3]&paged=$matches[4]&post_type='.$post_type, 'top' );


				add_rewrite_rule( $slug.'/date/([0-9]{4})/([0-9]{1,2})/([0-9]{1,2})/?$', 'index.php?year=$matches[1]&monthnum=$matches[2]&day=$matches[3]&post_type='.$post_type, 'top' );


				add_rewrite_rule( $slug.'/date/([0-9]{4})/([0-9]{1,2})/feed/(feed|rdf|rss|rss2|atom)/?$', 'index.php?year=$matches[1]&monthnum=$matches[2]&feed=$matches[3]&post_type='.$post_type, 'top' );


				add_rewrite_rule( $slug.'/date/([0-9]{4})/([0-9]{1,2})/(feed|rdf|rss|rss2|atom)/?$', 'index.php?year=$matches[1]&monthnum=$matches[2]&feed=$matches[3]&post_type='.$post_type, 'top' );


				add_rewrite_rule( $slug.'/date/([0-9]{4})/([0-9]{1,2})/page/?([0-9]{1,})/?$', 'index.php?year=$matches[1]&monthnum=$matches[2]&paged=$matches[3]&post_type='.$post_type, 'top' );


				add_rewrite_rule( $slug.'/date/([0-9]{4})/([0-9]{1,2})/?$', 'index.php?year=$matches[1]&monthnum=$matches[2]&post_type='.$post_type, 'top' );


				add_rewrite_rule( $slug.'/date/([0-9]{4})/feed/(feed|rdf|rss|rss2|atom)/?$', 'index.php?year=$matches[1]&feed=$matches[2]&post_type='.$post_type, 'top' );


				add_rewrite_rule( $slug.'/date/([0-9]{4})/(feed|rdf|rss|rss2|atom)/?$', 'index.php?year=$matches[1]&feed=$matches[2]&post_type='.$post_type, 'top' );


				add_rewrite_rule( $slug.'/date/([0-9]{4})/page/?([0-9]{1,})/?$', 'index.php?year=$matches[1]&paged=$matches[2]&post_type='.$post_type, 'top' );


				add_rewrite_rule( $slug.'/date/([0-9]{4})/?$', 'index.php?year=$matches[1]&post_type='.$post_type, 'top' );


				add_rewrite_rule( $slug.'/author/([^/]+)/?$', 'index.php?author_name=$matches[1]&post_type='.$post_type, 'top' );


			}








		endforeach;


	}





	/**


	 *


	 * registered_post_type


	 *  ** add rewrite tag for Custom Post Type.


	 * @version 1.1


	 * @since 0.9


	 *


	 */





	public function registered_post_type( $post_type, $args ) {





		global $wp_post_types, $wp_rewrite, $wp;





		if( $args->_builtin or !$args->publicly_queryable or !$args->show_ui ){


			return false;


		}


		$permalink = get_option( $post_type.'_structure' );





		if( !$permalink ) {


			$permalink = $this->default_structure;


		}





		$permalink = '%'.$post_type.'_slug%'.$permalink;


		$permalink = str_replace( '%postname%', '%'.$post_type.'%', $permalink );





		add_rewrite_tag( '%'.$post_type.'_slug%', '('.$args->rewrite['slug'].')','post_type='.$post_type.'&slug=' );





		$taxonomies = get_taxonomies( array("show_ui" => true, "_builtin" => false), 'objects' );


		foreach ( $taxonomies as $taxonomy => $objects ):


			$wp_rewrite->add_rewrite_tag( "%$taxonomy%", '(.+?)', "$taxonomy=" );


		endforeach;





		$permalink = trim($permalink, "/" );


		$rewrite_args = $args->rewrite;


		$rewrite_args["walk_dirs"] = false;


		add_permastruct( $post_type, $permalink, $rewrite_args);





	}

















	/**


	 *


	 * fix attachment output


	 *


	 * @version 1.0


	 * @since 0.8.2


	 *


	 */





	public function attachment_link( $link , $postID ) {


		$post = get_post( $postID );


		if (!$post->post_parent){


			return $link;


		}


		$post_parent = get_post( $post->post_parent );


		$permalink = get_option( $post_parent->post_type.'_structure' );


		$post_type = get_post_type_object( $post_parent->post_type );





		if( $post_type->_builtin == false ) {


			if(strpos( $permalink, "%postname%" ) < strrpos( $permalink, "%post_id%" ) && strrpos( $permalink, "attachment/" ) === FALSE ){


				$link = str_replace($post->post_name , "attachment/".$post->post_name, $link);


			}


		}





		return $link;


	}











	/**


	 *


	 * Fix permalinks output.


	 *


	 * @param String $post_link


	 * @param Object $post 投稿情報


	 * @param String $leavename 記事編集画面でのみ渡される


	 *


	 * @version 2.0


	 *


	 */


	public function post_type_link( $post_link, $post, $leavename ) {





		global $wp_rewrite;


		$draft_or_pending = isset( $post->post_status ) && in_array( $post->post_status, array( 'draft', 'pending', 'auto-draft' ) );


		if( $draft_or_pending and !$leavename )


			return $post_link;





		$post_type = $post->post_type;


		$permalink = $wp_rewrite->get_extra_permastruct( $post_type );


		$permalink = str_replace( '%post_id%', $post->ID, $permalink );


		$permalink = str_replace( '%'.$post_type.'_slug%', get_post_type_object( $post_type )->rewrite['slug'], $permalink );








		$parentsDirs = "";


		if( !$leavename ){


			$postId = $post->ID;


			while ($parent = get_post($postId)->post_parent) {


				$parentsDirs = get_post($parent)->post_name."/".$parentsDirs;


				$postId = $parent;


			}


		}





		$permalink = str_replace( '%'.$post_type.'%', $parentsDirs.'%'.$post_type.'%', $permalink );





		if( !$leavename ){


			$permalink = str_replace( '%'.$post_type.'%', $post->post_name, $permalink );


		}





		//%post_id%/attachment/%attachement_name%;


		//画像の編集ページでのリンク


		if( isset($_GET["post"]) && $_GET["post"] != $post->ID ) {


			$parent_structure = trim(get_option( $post->post_type.'_structure' ), "/");


			if( "%post_id%" == $parent_structure or "%post_id%" == array_pop( explode( "/", $parent_structure ) ) ) {


				$permalink = $permalink."/attachment/";


			};


		}





		$taxonomies = get_taxonomies( array('show_ui' => true),'objects' );





		//%taxnomomy% -> parent/child


		//運用でケアすべきかも。


		foreach ( $taxonomies as $taxonomy => $objects ) {


			if ( strpos($permalink, "%$taxonomy%") !== false ) {


				$terms = wp_get_post_terms( $post->ID, $taxonomy, array('orderby' => 'term_id'));


				if ( $terms and count($terms) > 1 ) {


					if(reset($terms)->parent == 0){





						$keys = array_keys($terms);


						$term = $terms[$keys[1]]->slug;


						if ( $terms[$keys[0]]->term_id == $terms[$keys[1]]->parent ) {


							$term = $this->get_taxonomy_parents( $terms[$keys[1]]->parent,$taxonomy, false, '/', true ) . $term;


						}


					}else{


						$keys = array_keys($terms);


						$term = $terms[$keys[0]]->slug;


						if ( $terms[$keys[1]]->term_id == $terms[$keys[0]]->parent ) {


							$term = $this->get_taxonomy_parents( $terms[$keys[0]]->parent,$taxonomy, false, '/', true ) . $term;


						}


					}


				}else if( $terms ){





					$term_obj = array_shift($terms);


					$term = $term_obj->slug;





					if(isset($term_obj->parent) and $term_obj->parent != 0) {


						$term = $this->get_taxonomy_parents( $term_obj->parent,$taxonomy, false, '/', true ) . $term;


					}


				}





				if( isset($term) ) {


					$permalink = str_replace( "%$taxonomy%", $term, $permalink );


				}


			}


		}





		$user = get_userdata( $post->post_author );


		if(isset($user->user_nicename)) {


			$permalink = str_replace( "%author%", $user->user_nicename, $permalink );


		}





		$post_date = strtotime( $post->post_date );


		$permalink = str_replace( "%year%", 	date("Y",$post_date), $permalink );


		$permalink = str_replace( "%monthnum%", date("m",$post_date), $permalink );


		$permalink = str_replace( "%day%", 		date("d",$post_date), $permalink );


		$permalink = str_replace( "%hour%", 	date("H",$post_date), $permalink );


		$permalink = str_replace( "%minute%", 	date("i",$post_date), $permalink );


		$permalink = str_replace( "%second%", 	date("s",$post_date), $permalink );








		$permalink = home_url()."/".user_trailingslashit( $permalink );


		$permalink = str_replace("//", "/", $permalink);


		$permalink = str_replace(":/", "://", $permalink);


		return $permalink;


	}











	/**


	 *


	 * wp_get_archives fix for custom post


	 * Ex:wp_get_archives('&post_type='.get_query_var( 'post_type' ));


	 * @version 2.0


	 *


	 */





	public $get_archives_where_r;





	// function modified by [steve]


	public function getarchives_where( $where, $r ) {


		$this->get_archives_where_r = $r;


		if ( isset($r['post_type']) ) {


			$where = str_replace( '\'post\'', '\'' . $r['post_type'] . '\'', $where );


		}





		if(isset($r['taxonomy']) && is_array($r['taxonomy']) ){


			global $wpdb;


			$where = $where . " AND $wpdb->term_taxonomy.taxonomy = '".$r['taxonomy']['name']."' AND $wpdb->term_taxonomy.term_id = '".$r['taxonomy']['termid']."'";


		}





		return $where;


	}











	//function added by [steve]


	/**


	 *


	 * get_archive_join


	 * @author Steve


	 * @since 0.8


	 * @version 1.0


	 *


	 *


	 */


	public function getarchives_join( $join, $r ) {


		global $wpdb;


		$this->get_archives_where_r = $r;


		if(isset($r['taxonomy']) && is_array($r['taxonomy']) )


		$join = $join . " INNER JOIN $wpdb->term_relationships ON ($wpdb->posts.ID = $wpdb->term_relationships.object_id) INNER JOIN $wpdb->term_taxonomy ON ($wpdb->term_relationships.term_taxonomy_id = $wpdb->term_taxonomy.term_taxonomy_id)";





		return $join;


	}











	/**


	 *


	 * get_arcihves_link


	 * @version 2.1


	 *


	 */


	public function get_archives_link( $link ) {


		if(!isset($this->get_archives_where_r['post_type'])) {


			return $link;


		}


		$c = isset($this->get_archives_where_r['taxonomy']) && is_array($this->get_archives_where_r['taxonomy']) ? $this->get_archives_where_r['taxonomy'] : "";  //[steve]


		$t =  $this->get_archives_where_r['post_type'];








		$this->get_archives_where_r['post_type'] = isset($this->get_archives_where_r['post_type_slug']) ? $this->get_archives_where_r['post_type_slug'] : $t; // [steve] [*** bug fixing]





		if (isset($this->get_archives_where_r['post_type'])  and  $this->get_archives_where_r['type'] != 'postbypost'){


			$blog_url = rtrim( get_bloginfo("url") ,'/');





			//remove .ext


			$str = preg_replace("/\.[a-z,_]*/","",get_option("permalink_structure"));





			if($str = rtrim( preg_replace("/%[a-z,_]*%/","",$str) ,'/')) { // /archive/%post_id%


				$ret_link = str_replace($str, '/'.'%link_dir%', $link);


			}else{


				$blog_url = preg_replace('/https?:\/\//', '', $blog_url);


				$ret_link = str_replace($blog_url,$blog_url.'/'.'%link_dir%',$link);


			}





			$post_type = get_post_type_object( $this->get_archives_where_r['post_type'] );


			if(empty($c) ){    // [steve]


				if(isset( $post_type->rewrite["slug"] )) {


					$link_dir = $post_type->rewrite["slug"];


				}


				else{


					$link_dir = $this->get_archives_where_r['post_type'];


				}


			}


			else{   // [steve]


				$c['name'] = ($c['name'] == 'category' && get_option('category_base')) ? get_option('category_base') : $c['name'];


				$link_dir = $post_type->rewrite["slug"]."/".$c['name']."/".$c['termslug'];


			}





			if(!strstr($link,'/date/')){


				$link_dir = $link_dir .'/date';


			}





			$ret_link = str_replace('%link_dir%',$link_dir,$ret_link);


		}else {


			$ret_link = $link;


		}


		$this->get_archives_where_r['post_type'] = $t;	// [steve] reverting post_type to previous value





		return $ret_link;


	}











	/**


	 *


	 * Add rewrite rules for custom taxonomies.


	 * @since 0.6


	 * @version 2.1


	 *


	 */


	public function add_tax_rewrite_rules() {


		if(get_option('no_taxonomy_structure')) {


			return false;


		}








		global $wp_rewrite;


		$taxonomies = get_taxonomies(array( '_builtin' => false));


		$taxonomies['category'] = 'category';





		if(empty($taxonomies)) {


			return false;


		}





		foreach ($taxonomies as $taxonomy) :


			$taxonomyObject = get_taxonomy($taxonomy);


			$post_types = $taxonomyObject->object_type;





			foreach ($post_types as $post_type):


				$post_type_obj = get_post_type_object($post_type);


				$slug = $post_type_obj->rewrite['slug'];


				if(!$slug) {


					$slug = $post_type;


				}





				if(is_string($post_type_obj->has_archive)) {


					$slug = $post_type_obj->has_archive;


				};





				if ( $taxonomy == 'category' ){


					$taxonomypat = ($cb = get_option('category_base')) ? $cb : $taxonomy;


					$tax = 'category_name';


				} else {


					// Edit by [Xiphe]


					if (isset($taxonomyObject->rewrite['slug'])) {


						$taxonomypat = $taxonomyObject->rewrite['slug'];


					} else {


						$taxonomypat = $taxonomy;


					}


					// [Xiphe] stop





					$tax = $taxonomy;


				}








				//add taxonomy slug


				add_rewrite_rule( $slug.'/'.$taxonomypat.'/(.+?)/page/?([0-9]{1,})/?$', 'index.php?'.$tax.'=$matches[1]&paged=$matches[2]', 'top' );


				add_rewrite_rule( $slug.'/'.$taxonomypat.'/(.+?)/feed/(feed|rdf|rss|rss2|atom)/?$', 'index.php?'.$tax.'=$matches[1]&feed=$matches[2]', 'top' );


				add_rewrite_rule( $slug.'/'.$taxonomypat.'/(.+?)/(feed|rdf|rss|rss2|atom)/?$', 'index.php?'.$tax.'=$matches[1]&feed=$matches[2]', 'top' );


				add_rewrite_rule( $slug.'/'.$taxonomypat.'/(.+?)/?$', 'index.php?'.$tax.'=$matches[1]', 'top' );  // modified by [steve] [*** bug fixing]





				// below rules were added by [steve]


				add_rewrite_rule( $taxonomypat.'/(.+?)/date/([0-9]{4})/([0-9]{1,2})/?$', 'index.php?'.$tax.'=$matches[1]&year=$matches[2]&monthnum=$matches[3]&post_type='.$post_type, 'top' );


				add_rewrite_rule( $taxonomypat.'/(.+?)/date/([0-9]{4})/([0-9]{1,2})/page/?([0-9]{1,})/?$', 'index.php?'.$tax.'=$matches[1]&year=$matches[2]&monthnum=$matches[3]&paged=$matches[4]&post_type='.$post_type, 'top' );


				add_rewrite_rule( $slug.'/'.$taxonomypat.'/(.+?)/date/([0-9]{4})/([0-9]{1,2})/?$', 'index.php?'.$tax.'=$matches[1]&year=$matches[2]&monthnum=$matches[3]&post_type='.$post_type, 'top' );


				add_rewrite_rule( $slug.'/'.$taxonomypat.'/(.+?)/date/([0-9]{4})/([0-9]{1,2})/page/?([0-9]{1,})/?$', 'index.php?'.$tax.'=$matches[1]&year=$matches[2]&monthnum=$matches[3]&paged=$matches[4]&post_type='.$post_type, 'top' );





			endforeach;


		endforeach;


	}














	/**


	 *


	 * Fix taxonomy link outputs.


	 * @since 0.6


	 * @version 1.0


	 *


	 */


	public function term_link( $termlink, $term, $taxonomy ) {


		if( get_option('no_taxonomy_structure') ) {


			return  $termlink;


		}





		$taxonomy = get_taxonomy($taxonomy);


		if( $taxonomy->_builtin )


			return $termlink;





		if( empty($taxonomy) )


			return $termlink;





		$wp_home = rtrim( home_url(), '/' );





		$post_type = $taxonomy->object_type[0];


		$slug = get_post_type_object($post_type)->rewrite['slug'];


		$with_front = get_post_type_object($post_type)->rewrite['with_front'];





		//$termlink = str_replace( $term->slug.'/', $this->get_taxonomy_parents( $term->term_id,$taxonomy->name, false, '/', true ), $termlink );





		//拡張子を削除。


		$str = array_shift(explode(".", get_option("permalink_structure")));


		$str = rtrim( preg_replace( "/%[a-z_]*%/", "" ,$str) ,'/' );//remove with front


		$termlink = str_replace($str."/", "/", $termlink );





		if( $with_front === false ) {


			$str = "";


		}


		$slug = $str."/".$slug;





		$termlink = str_replace( $wp_home, $wp_home.$slug, $termlink );


		$termlink = str_replace( $term->slug.'/', $this->get_taxonomy_parents( $term->term_id,$taxonomy->name, false, '/', true ), $termlink );


		return $termlink;


	}





	/**


	 *


	 * Fix taxonomy = parent/child => taxonomy => child


	 * @since 0.9.3


	 *


	 */


	public function parse_request($obj) {


		$taxes = get_taxonomies(array( '_builtin' => false));


		foreach ($taxes as $key => $tax) {


			if(isset($obj->query_vars[$tax])) {


				if(strpos( $obj->query_vars[$tax] ,"/") !== false ) {


					$obj->query_vars[$tax] = array_pop(explode("/", $obj->query_vars[$tax]));


				}


			}


		}


	}











	/**


	 *


	 * load textdomain


	 * @since 0.6.2


	 *


	 */


	public function load_textdomain() {


		load_plugin_textdomain('cptp',false,'custom-post-type-permalinks/language');


	}











	/**


	 *


	 * Add hook flush_rules


	 * @since 0.7.9


	 *


	 */


	public function update_rules() {





		$post_types = get_post_types( array('_builtin'=>false, 'publicly_queryable'=>true, 'show_ui' => true) );


		$type_count = count($post_types);


		$i = 0;


		foreach ($post_types as $post_type):


			add_action('update_option_'.$post_type.'_structure',array(&$this,'queue_flush_rules'),10,2);


		endforeach;


		add_action('update_option_no_taxonomy_structure',array(&$this,'queue_flush_rules'),10,2);


	}











	/**


	 * Flush rules


	 *


	 * @since 0.7.9


	 *


	 */





	public function queue_flush_rules(){


		update_option( "queue_flush_rules", 1 );


	}











	/**


	 *


	 * Setting Init


	 * @since 0.7


	 *


	 */


	public function settings_api_init() {


		add_settings_section('cptp_setting_section',


			__("Permalink Setting for custom post type",'cptp'),


			array(&$this,'setting_section_callback_function'),


			'permalink'


		);





		$post_types = get_post_types( array('_builtin'=>false, 'publicly_queryable'=>true, 'show_ui' => true) );


		foreach ($post_types as $post_type):


			if(isset($_POST['submit']) and isset($_POST['_wp_http_referer'])){


				if( strpos($_POST['_wp_http_referer'],'options-permalink.php') !== FALSE ) {





					$structure = trim(esc_attr($_POST[$post_type.'_structure']));#get setting





					#default permalink structure


					if( !$structure )


						$structure = $this->default_structure;





					$structure = str_replace('//','/','/'.$structure);# first "/"





					#last "/"


					$lastString = substr(trim(esc_attr($_POST['permalink_structure'])),-1);


					$structure = rtrim($structure,'/');





					if ( $lastString == '/')


						$structure = $structure.'/';





					update_option($post_type.'_structure', $structure );


				}


			}





			add_settings_field($post_type.'_structure',


				$post_type,


				array(&$this,'setting_structure_callback_function'),


				'permalink',


				'cptp_setting_section',


				$post_type.'_structure'


			);





			register_setting('permalink',$post_type.'_structure');


		endforeach;





		add_settings_field(


			'no_taxonomy_structure',


			__("Use custom permalink of custom taxonomy archive.",'cptp'),


			array(&$this,'setting_no_tax_structure_callback_function'),


			'permalink',


			'cptp_setting_section'


		);





		register_setting('permalink','no_taxonomy_structure');





		if(isset($_POST['submit']) && isset($_POST['_wp_http_referer']) && strpos($_POST['_wp_http_referer'],'options-permalink.php') !== FALSE ) {





			if(!isset($_POST['no_taxonomy_structure'])){


				$set = true;


			}else {


				$set = false;


			}


			update_option('no_taxonomy_structure', $set);


		}


	}





	public function setting_section_callback_function() {


		?>


			<p><?php _e("Setting permalinks of custom post type.",'cptp');?><br />


			<?php _e("The tags you can use is WordPress Structure Tags and '%\"custom_taxonomy_slug\"%'. (e.g. %actors%)",'cptp');?><br />


			<?php _e("%\"custom_taxonomy_slug\"% is replaced the taxonomy's term.'.",'cptp');?></p>





			<p><?php _e("Presence of the trailing '/' is unified into a standard permalink structure setting.",'cptp');?>


			<?php _e("If you don't entered permalink structure, permalink is configured /%postname%/'.",'cptp');?>


			</p>


		<?php


	}





	public function setting_structure_callback_function(  $option  ) {


		$post_type = str_replace('_structure',"" ,$option);


		$slug = get_post_type_object($post_type)->rewrite['slug'];


		$with_front = get_post_type_object($post_type)->rewrite['with_front'];





		$value = get_option($option);


		if( !$value )


			$value = $this->default_structure;





		global $wp_rewrite;


		$front = substr( $wp_rewrite->front, 1 );


		if( $front and $with_front ) {


			$slug = $front.$slug;


		}





		echo '<code>'.home_url().'/'.$slug.'</code> <input name="'.$option.'" id="'.$option.'" type="text" class="regular-text code" value="' . $value .'" />';


	}





	public function setting_no_tax_structure_callback_function(){


		echo '<input name="no_taxonomy_structure" id="no_taxonomy_structure" type="checkbox" value="1" class="code" ' . checked( false, get_option('no_taxonomy_structure'),false) . ' /> ';


		$txt = __("If you check,The custom taxonomy's permalinks is <code>%s/post_type/taxonomy/term</code>.","cptp");


		printf($txt , home_url());


	}











	/**


	 *


	 * enqueue CSS and JS


	 * @since 0.8.5


	 *


	 */


	public function enqueue_css_js() {


		wp_enqueue_style('wp-pointer');


		wp_enqueue_script('wp-pointer');


	}











	/**


	 *


	 * add js for pointer


	 * @since 0.8.5


	 */


	public function pointer_js() {


		if(!is_network_admin()) {


			$dismissed = explode(',', get_user_meta( get_current_user_id(), 'dismissed_wp_pointers', true ));


			if(array_search('cptp_pointer0871', $dismissed) === false){


				$content = __("<h3>Custom Post Type Permalinks</h3><p>From <a href='options-permalink.php'>Permalinks</a>, set a custom permalink for each post type.</p>", "cptp");


			?>


				<script type="text/javascript">


				jQuery(function($) {





					$("#menu-settings .wp-has-submenu").pointer({


						content: "<?php echo $content;?>",


						position: {"edge":"left","align":"center"},


						close: function() {


							$.post('admin-ajax.php', {


								action:'dismiss-wp-pointer',


								pointer: 'cptp_pointer0871'


							})





						}


					}).pointer("open");


				});


				</script>


			<?php


			}


		}


	}


} 


$custom_post_type_permalinks = new Custom_Post_Type_Permalinks;


$custom_post_type_permalinks = apply_filters('custom_post_type_permalinks', $custom_post_type_permalinks);


$custom_post_type_permalinks->add_hook();


?>